@@ -1,38 +1,46 @@
 import json
 import logging
-import operator
 import os
-from collections import OrderedDict, defaultdict
+from collections import defaultdict
 from itertools import islice
+from typing import Dict
 
 from multiqc import config
+from multiqc.base_module import BaseMultiqcModule, ModuleNoSamplesFound
 from multiqc.plots import bargraph, table
-from multiqc.modules.base_module import BaseMultiqcModule
+from multiqc.plots.table_object import ColumnDict, ValueT
 
 log = logging.getLogger(__name__)
 
 
 class MultiqcModule(BaseMultiqcModule):
+    """
+    There are two versions of this software: `bcl2fastq` for MiSeq and HiSeq
+    sequencing systems running RTA versions earlier than 1.8, and `bcl2fastq2` for
+    Illumina sequencing systems running RTA version 1.18.54 and above. This module
+    currently only covers output from the latter.
+    """
+
     def __init__(self):
         # Initialise the parent object
         super(MultiqcModule, self).__init__(
             name="bcl2fastq",
             anchor="bcl2fastq",
             href="https://support.illumina.com/sequencing/sequencing_software/bcl2fastq-conversion-software.html",
-            info="can be used to both demultiplex data and convert BCL files to FASTQ file formats for downstream analysis.",
+            info="Demultiplexes data and converts BCL files to FASTQ file formats for downstream analysis.",
             # Can't find a DOI // doi=
         )
 
         # Gather data from all json files
-        self.bcl2fastq_data = dict()
+        self.bcl2fastq_data: Dict[str, Dict[str, dict]] = dict()
         for f in self.find_log_files("bcl2fastq"):
             self.parse_file_as_json(f)
 
         # Collect counts by lane and sample (+source_files)
-        self.bcl2fastq_bylane = dict()
-        self.bcl2fastq_bysample = dict()
-        self.bcl2fastq_bysample_lane = dict()
-        self.source_files = dict()
+        self.bcl2fastq_bylane: dict = dict()
+        self.bcl2fastq_bysample: dict = dict()
+        self.bcl2fastq_bysample_lane: dict = dict()
+        self.source_files: Dict[str, list[str]] = dict()
         self.split_data_by_lane_and_sample()
 
         # Filter to strip out ignored sample names
@@ -42,16 +50,17 @@ class MultiqcModule(BaseMultiqcModule):
 
         # Return with Warning if no files are found
         if len(self.bcl2fastq_bylane) == 0 and len(self.bcl2fastq_bysample) == 0:
-            raise UserWarning
+            raise ModuleNoSamplesFound
 
         # Print source files
-        for s in self.source_files.keys():
-            self.add_data_source(
-                s_name=s,
-                source=",".join(list(set(self.source_files[s]))),
-                module="bcl2fastq",
-                section="bcl2fastq-bysample",
-            )
+        for s, paths in self.source_files.items():
+            for path in paths:
+                self.add_data_source(
+                    s_name=s,
+                    path=path,
+                    module="bcl2fastq",
+                    section="bcl2fastq-bysample",
+                )
 
         # Add sample counts to general stats table
         self.add_general_stats()
@@ -69,10 +78,11 @@ class MultiqcModule(BaseMultiqcModule):
         )
 
         # Add section for counts by lane
-        cats = OrderedDict()
-        cats["perfect"] = {"name": "Perfect Index Reads"}
-        cats["imperfect"] = {"name": "Mismatched Index Reads"}
-        cats["undetermined"] = {"name": "Undetermined Reads"}
+        cats = {
+            "perfect": {"name": "Perfect Index Reads"},
+            "imperfect": {"name": "Mismatched Index Reads"},
+            "undetermined": {"name": "Undetermined Reads"},
+        }
         self.add_section(
             name="Clusters by lane",
             anchor="bcl2fastq-bylane",
@@ -93,10 +103,10 @@ class MultiqcModule(BaseMultiqcModule):
 
         # Add section for counts by sample
         # get cats for per-lane tab
-        lcats = set()
+        lcats_set: set[str] = set()
         for s_name in self.bcl2fastq_bysample_lane:
-            lcats.update(self.bcl2fastq_bysample_lane[s_name].keys())
-        lcats = sorted(list(lcats))
+            lcats_set.update(self.bcl2fastq_bysample_lane[s_name].keys())
+        lcats = sorted(list(lcats_set))
         self.add_section(
             name="Clusters by sample",
             anchor="bcl2fastq-bysample",
@@ -120,7 +130,7 @@ class MultiqcModule(BaseMultiqcModule):
         # Add section with undetermined barcodes
         self.add_section(
             name="Undetermined barcodes by lane",
-            anchor="undetermine_by_lane",
+            anchor="undetermined_by_lane",
             description="Count of the top twenty most abundant undetermined barcodes by lanes",
             plot=bargraph.plot(
                 self.get_bar_data_from_undetermined(self.bcl2fastq_bylane),
@@ -128,10 +138,8 @@ class MultiqcModule(BaseMultiqcModule):
                 {
                     "id": "bcl2fastq_undetermined",
                     "title": "bcl2fastq: Undetermined barcodes by lane",
-                    "ylab": "Count",
-                    "tt_percentages": False,
-                    "use_legend": True,
-                    "tt_suffix": "reads",
+                    "ylab": "Reads",
+                    "sort_samples": False,  # keep top barcode on top
                 },
             ),
         )
@@ -140,22 +148,24 @@ class MultiqcModule(BaseMultiqcModule):
         try:
             content = json.loads(f["f"])
         except ValueError:
-            log.warning("Could not parse file as json: {}".format(f["fn"]))
+            log.warning(f"Could not parse file as json: {f['fn']}")
             return
 
         # Clean / prepend directories to sample names
         runId = self.clean_s_name(content["RunId"], f)
 
+        # Superfluous function call to confirm that it is used in this module
+        # Replace None with actual version if it is available
+        self.add_software_version(None, runId)
+
         if runId not in self.bcl2fastq_data:
             self.bcl2fastq_data[runId] = dict()
-        run_data = self.bcl2fastq_data[runId]
+        run_data: Dict[str, Dict] = self.bcl2fastq_data[runId]
         for conversionResult in content.get("ConversionResults", []):
-            l = conversionResult["LaneNumber"]
-            lane = "L{}".format(conversionResult["LaneNumber"])
+            lane_num = conversionResult["LaneNumber"]
+            lane = f"L{conversionResult['LaneNumber']}"
             if lane in run_data:
-                log.debug(
-                    "Duplicate runId/lane combination found! Overwriting: {}".format(self.prepend_runid(runId, lane))
-                )
+                log.debug(f"Duplicate runId/lane combination found! Overwriting: {self.prepend_runid(runId, lane)}")
             run_data[lane] = {
                 "total": 0,
                 "total_yield": 0,
@@ -170,7 +180,7 @@ class MultiqcModule(BaseMultiqcModule):
             # Add undetermined barcodes
             unknown_barcode = dict()
             for lane_data in content.get("UnknownBarcodes", list()):
-                if lane_data["Lane"] == l:
+                if lane_data["Lane"] == lane_num:
                     unknown_barcode = lane_data["Barcodes"]
                     break
             run_data[lane]["unknown_barcodes"] = unknown_barcode
@@ -179,7 +189,7 @@ class MultiqcModule(BaseMultiqcModule):
                 if demuxResult["SampleName"] == demuxResult["SampleId"]:
                     sample = demuxResult["SampleName"]
                 else:
-                    sample = "{}-{}".format(demuxResult["SampleId"], demuxResult["SampleName"])
+                    sample = f"{demuxResult['SampleId']}-{demuxResult['SampleName']}"
                 sample = self.clean_s_name(sample, f)
                 if sample in run_data[lane]["samples"]:
                     log.debug(
@@ -197,10 +207,10 @@ class MultiqcModule(BaseMultiqcModule):
                 }
                 # simplify the population of dictionaries
                 lsample = run_data[lane]["samples"][sample]
-                for r in range(1, 5):
-                    lsample["R{}_yield".format(r)] = 0
-                    lsample["R{}_Q30".format(r)] = 0
-                    lsample["R{}_trimmed_bases".format(r)] = 0
+                for r_num in range(1, 5):
+                    lsample[f"R{r_num}_yield"] = 0
+                    lsample[f"R{r_num}_Q30"] = 0
+                    lsample[f"R{r_num}_trimmed_bases"] = 0
                 rlane["total"] += demuxResult["NumberReads"]
                 rlane["total_yield"] += demuxResult["Yield"]
                 lsample["total"] += demuxResult["NumberReads"]
@@ -209,24 +219,24 @@ class MultiqcModule(BaseMultiqcModule):
                     rlane["perfectIndex"] += indexMetric["MismatchCounts"]["0"]
                     lsample["perfectIndex"] += indexMetric["MismatchCounts"]["0"]
                 for readMetric in demuxResult.get("ReadMetrics", []):
-                    r = readMetric["ReadNumber"]
+                    r_num = readMetric["ReadNumber"]
                     rlane["yieldQ30"] += readMetric["YieldQ30"]
                     rlane["qscore_sum"] += readMetric["QualityScoreSum"]
                     lsample["yieldQ30"] += readMetric["YieldQ30"]
                     lsample["qscore_sum"] += readMetric["QualityScoreSum"]
-                    lsample["R{}_yield".format(r)] += readMetric["Yield"]
-                    lsample["R{}_Q30".format(r)] += readMetric["YieldQ30"]
-                    lsample["R{}_trimmed_bases".format(r)] += readMetric["TrimmedBases"]
+                    lsample[f"R{r_num}_yield"] += readMetric["Yield"]
+                    lsample[f"R{r_num}_Q30"] += readMetric["YieldQ30"]
+                    lsample[f"R{r_num}_trimmed_bases"] += readMetric["TrimmedBases"]
                 # Remove unpopulated read keys
-                for r in range(1, 5):
+                for r_num in range(1, 5):
                     if (
-                        not lsample["R{}_yield".format(r)]
-                        and not lsample["R{}_Q30".format(r)]
-                        and not lsample["R{}_trimmed_bases".format(r)]
+                        not lsample[f"R{r_num}_yield"]
+                        and not lsample[f"R{r_num}_Q30"]
+                        and not lsample[f"R{r_num}_trimmed_bases"]
                     ):
-                        lsample.pop("R{}_yield".format(r))
-                        lsample.pop("R{}_Q30".format(r))
-                        lsample.pop("R{}_trimmed_bases".format(r))
+                        lsample.pop(f"R{r_num}_yield")
+                        lsample.pop(f"R{r_num}_Q30")
+                        lsample.pop(f"R{r_num}_trimmed_bases")
             undeterminedYieldQ30 = 0
             undeterminedQscoreSum = 0
             undeterminedTrimmedBases = 0
@@ -245,21 +255,23 @@ class MultiqcModule(BaseMultiqcModule):
                 }
 
         # Calculate Percents and averages
-        for lane_id, lane in run_data.items():
+        for lane_id, lane_data in run_data.items():
             try:
-                lane["percent_Q30"] = (float(lane["yieldQ30"]) / float(lane["total_yield"])) * 100.0
+                lane_data["percent_Q30"] = (float(lane_data["yieldQ30"]) / float(lane_data["total_yield"])) * 100.0
             except ZeroDivisionError:
-                lane["percent_Q30"] = "NA"
+                lane_data["percent_Q30"] = "NA"
             try:
-                lane["percent_perfectIndex"] = (float(lane["perfectIndex"]) / float(lane["total"])) * 100.0
+                lane_data["percent_perfectIndex"] = (
+                    float(lane_data["perfectIndex"]) / float(lane_data["total"])
+                ) * 100.0
             except ZeroDivisionError:
-                lane["percent_perfectIndex"] = "NA"
+                lane_data["percent_perfectIndex"] = "NA"
             try:
-                lane["mean_qscore"] = float(lane["qscore_sum"]) / float(lane["total_yield"])
+                lane_data["mean_qscore"] = float(lane_data["qscore_sum"]) / float(lane_data["total_yield"])
             except ZeroDivisionError:
-                lane["mean_qscore"] = "NA"
+                lane_data["mean_qscore"] = "NA"
 
-            for sample_id, sample in lane["samples"].items():
+            for sample_id, sample in lane_data["samples"].items():
                 try:
                     sample["percent_Q30"] = (float(sample["yieldQ30"]) / float(sample["total_yield"])) * 100.0
                 except ZeroDivisionError:
@@ -274,8 +286,8 @@ class MultiqcModule(BaseMultiqcModule):
                     sample["mean_qscore"] = "NA"
 
     def split_data_by_lane_and_sample(self):
-        for run_id, r in self.bcl2fastq_data.items():
-            for lane_id, lane in r.items():
+        for run_id, run_dict in self.bcl2fastq_data.items():
+            for lane_id, lane in run_dict.items():
                 uniqLaneName = self.prepend_runid(run_id, lane_id)
                 self.bcl2fastq_bylane[uniqLaneName] = {
                     "total": lane["total"],
@@ -287,7 +299,7 @@ class MultiqcModule(BaseMultiqcModule):
                     "percent_Q30": lane["percent_Q30"],
                     "percent_perfectIndex": lane["percent_perfectIndex"],
                     "mean_qscore": lane["mean_qscore"],
-                    "unknown_barcodes": self.get_unknown_barcodes(lane["unknown_barcodes"]),
+                    "unknown_barcodes": lane["unknown_barcodes"],
                 }
                 for sample_id, sample in lane["samples"].items():
                     if sample_id not in self.bcl2fastq_bysample:
@@ -298,10 +310,10 @@ class MultiqcModule(BaseMultiqcModule):
                             "yieldQ30": 0,
                             "qscore_sum": 0,
                         }
-                        for r in range(1, 5):
-                            self.bcl2fastq_bysample[sample_id]["R{}_yield".format(r)] = 0
-                            self.bcl2fastq_bysample[sample_id]["R{}_Q30".format(r)] = 0
-                            self.bcl2fastq_bysample[sample_id]["R{}_trimmed_bases".format(r)] = 0
+                        for r_num in range(1, 5):
+                            self.bcl2fastq_bysample[sample_id][f"R{r_num}_yield"] = 0
+                            self.bcl2fastq_bysample[sample_id][f"R{r_num}_Q30"] = 0
+                            self.bcl2fastq_bysample[sample_id][f"R{r_num}_trimmed_bases"] = 0
                     s = self.bcl2fastq_bysample[sample_id]
                     s["total"] += sample["total"]
                     s["total_yield"] += sample["total_yield"]
@@ -309,11 +321,11 @@ class MultiqcModule(BaseMultiqcModule):
                     s["yieldQ30"] += sample["yieldQ30"]
                     s["qscore_sum"] += sample["qscore_sum"]
                     # Undetermined samples did not have R1 and R2 information
-                    for r in range(1, 5):
+                    for r_num in range(1, 5):
                         try:
-                            s["R{}_yield".format(r)] += sample["R{}_yield".format(r)]
-                            s["R{}_Q30".format(r)] += sample["R{}_Q30".format(r)]
-                            s["R{}_trimmed_bases".format(r)] += sample["R{}_trimmed_bases".format(r)]
+                            s[f"R{r_num}_yield"] += sample[f"R{r_num}_yield"]
+                            s[f"R{r_num}_Q30"] += sample[f"R{r_num}_Q30"]
+                            s[f"R{r_num}_trimmed_bases"] += sample[f"R{r_num}_trimmed_bases"]
                         except KeyError:
                             pass
                     try:
@@ -334,47 +346,35 @@ class MultiqcModule(BaseMultiqcModule):
                         self.source_files[sample_id].append(sample["filename"])
                 # Remove unpopulated read keys
                 for sample_id, sample in lane["samples"].items():
-                    for r in range(1, 5):
+                    for r_num in range(1, 5):
                         try:
                             if (
-                                not self.bcl2fastq_bysample[sample_id]["R{}_yield".format(r)]
-                                and not self.bcl2fastq_bysample[sample_id]["R{}_Q30".format(r)]
-                                and not self.bcl2fastq_bysample[sample_id]["R{}_trimmed_bases".format(r)]
+                                not self.bcl2fastq_bysample[sample_id][f"R{r_num}_yield"]
+                                and not self.bcl2fastq_bysample[sample_id][f"R{r_num}_Q30"]
+                                and not self.bcl2fastq_bysample[sample_id][f"R{r_num}_trimmed_bases"]
                             ):
-                                self.bcl2fastq_bysample[sample_id].pop("R{}_yield".format(r))
-                                self.bcl2fastq_bysample[sample_id].pop("R{}_Q30".format(r))
-                                self.bcl2fastq_bysample[sample_id].pop("R{}_trimmed_bases".format(r))
+                                self.bcl2fastq_bysample[sample_id].pop(f"R{r_num}_yield")
+                                self.bcl2fastq_bysample[sample_id].pop(f"R{r_num}_Q30")
+                                self.bcl2fastq_bysample[sample_id].pop(f"R{r_num}_trimmed_bases")
                         except KeyError:
                             pass
 
-    def get_unknown_barcodes(self, lane_unknown_barcode):
-        """Python 2.* dictionaries are not sorted.
-        This function return an `OrderedDict` sorted by barcode count.
-        """
-        try:
-            sorted_barcodes = OrderedDict(
-                sorted(lane_unknown_barcode.items(), key=operator.itemgetter(1), reverse=True)
-            )
-        except AttributeError:
-            sorted_barcodes = None
-        return sorted_barcodes
-
     def add_general_stats(self):
-        data = dict()
+        data: Dict[str, Dict[str, ValueT]] = dict()
         for sample_id, sample in self.bcl2fastq_bysample.items():
             percent_R_Q30 = dict()
-            for r in range(1, 5):
+            for r_num in range(1, 5):
                 # Zero division is possible
                 try:
-                    percent_R_Q30[r] = "{0:.1f}".format(
-                        float(100.0 * sample["R{}_Q30".format(r)] / sample["R{}_yield".format(r)])
+                    percent_R_Q30[r_num] = "{0:.1f}".format(
+                        float(100.0 * sample[f"R{r_num}_Q30"] / sample[f"R{r_num}_yield"])
                     )
                 except ZeroDivisionError:
-                    percent_R_Q30[r] = "0.0"
+                    percent_R_Q30[r_num] = "0.0"
                 except KeyError:
                     pass
             try:
-                perfect_percent = "{0:.1f}".format(float(100.0 * sample["perfectIndex"] / sample["total"]))
+                perfect_percent = f"{float(100.0 * sample['perfectIndex'] / sample['total']):.1f}"
             except ZeroDivisionError:
                 perfect_percent = "0.0"
 
@@ -383,40 +383,40 @@ class MultiqcModule(BaseMultiqcModule):
                 "total": sample["total"],
                 "perfectPercent": perfect_percent,
             }
-            for r in range(1, 5):
+            for r_num in range(1, 5):
                 try:
-                    data[sample_id]["percent_R{}_Q30".format(r)] = percent_R_Q30[r]
-                    data[sample_id]["R{}_trimmed_bases".format(r)] = sample["R{}_trimmed_bases".format(r)]
+                    data[sample_id][f"percent_R{r_num}_Q30"] = percent_R_Q30[r_num]
+                    data[sample_id][f"R{r_num}_trimmed_bases"] = sample[f"R{r_num}_trimmed_bases"]
                 except KeyError:
                     pass
 
-        headers = OrderedDict()
-        headers["total"] = {
-            "title": "{} Clusters".format(config.read_count_prefix),
-            "description": "Total number of reads for this sample as determined by bcl2fastq demultiplexing ({})".format(
-                config.read_count_desc
-            ),
-            "scale": "Blues",
-            "shared_key": "read_count",
-        }
-        headers["yieldQ30"] = {
-            "title": "Yield ({}) &ge; Q30".format(config.base_count_prefix),
-            "description": "Number of bases with a Phred score of 30 or higher ({})".format(config.base_count_desc),
-            "scale": "Greens",
-            "shared_key": "base_count",
+        headers: Dict[str, ColumnDict] = {
+            "total": {
+                "title": "Clusters",
+                "description": f"Total number of reads for this sample as determined by bcl2fastq demultiplexing ("
+                f"{config.read_count_desc})",
+                "scale": "Blues",
+                "shared_key": "read_count",
+            },
+            "yieldQ30": {
+                "title": "Yield ≥ Q30",
+                "description": f"Number of bases with a Phred score of 30 or higher ({config.base_count_desc})",
+                "scale": "Greens",
+                "shared_key": "base_count",
+            },
         }
         # If no data for a column, header will be automatically ignored
-        for r in range(1, 5):
-            headers["percent_R{}_Q30".format(r)] = {
-                "title": "% R{} Yield &ge; Q30".format(r),
-                "description": "Percent of bases in R{} with a Phred score of 30 or higher".format(r),
+        for r_num in range(1, 5):
+            headers[f"percent_R{r_num}_Q30"] = {
+                "title": f"R{r_num} yield ≥ Q30",
+                "description": f"Percent of bases in R{r_num} with a Phred score of 30 or higher",
                 "scale": "RdYlGn",
                 "max": 100,
                 "min": 0,
                 "suffix": "%",
             }
         headers["perfectPercent"] = {
-            "title": "% Perfect Index",
+            "title": "Perfect index",
             "description": "Percent of reads with perfect index (0 mismatches)",
             "max": 100,
             "min": 0,
@@ -424,21 +424,21 @@ class MultiqcModule(BaseMultiqcModule):
             "suffix": "%",
         }
         # If no data for a column, header will be automatically ignored
-        for r in range(1, 5):
-            hideCol = True
-            for s in data:
+        for r_num in range(1, 5):
+            hide_col = True
+            for sname in data:
                 try:
-                    if data[s]["R{}_trimmed_bases".format(r)] > 0:
-                        hideCol = False
+                    if data[sname][f"R{r_num}_trimmed_bases"] > 0:
+                        hide_col = False
                 except KeyError:
                     pass
             try:
-                headers["R{}_trimmed_bases".format(r)] = {
-                    "title": "{} R{} trimmed".format(config.base_count_prefix, r),
-                    "description": "Number of bases trimmed ({})".format(config.base_count_desc),
+                headers[f"R{r_num}_trimmed_bases"] = {
+                    "title": f"{config.base_count_prefix} R{r_num} trimmed",
+                    "description": f"Number of bases trimmed ({config.base_count_desc})",
                     "scale": "RdYlBu",
                     "modify": lambda x: x * 0.000001,
-                    "hidden": hideCol,
+                    "hidden": hide_col,
                 }
             except KeyError:
                 pass
@@ -446,75 +446,79 @@ class MultiqcModule(BaseMultiqcModule):
 
     def lane_stats_table(self):
         """Return a table with overview stats for each bcl2fastq lane for a single flow cell"""
-        headers = OrderedDict()
-        headers["total_yield"] = {
-            "title": "{} Total Yield".format(config.base_count_prefix),
-            "description": "Number of bases ({})".format(config.base_count_desc),
-            "scale": "Greens",
-            "shared_key": "base_count",
-        }
-        headers["total"] = {
-            "title": "{} Total Clusters".format(config.read_count_prefix),
-            "description": "Total number of clusters for this lane ({})".format(config.read_count_desc),
-            "scale": "Blues",
-            "shared_key": "read_count",
-        }
-        headers["percent_Q30"] = {
-            "title": "% bases &ge; Q30",
-            "description": "Percentage of bases with greater than or equal to Q30 quality score",
-            "suffix": "%",
-            "max": 100,
-            "min": 0,
-            "scale": "RdYlGn",
-        }
-        headers["mean_qscore"] = {
-            "title": "Mean Quality",
-            "description": "Average phred qualty score",
-            "min": 0,
-            "scale": "Spectral",
-        }
-        headers["percent_perfectIndex"] = {
-            "title": "% Perfect Index",
-            "description": "Percent of reads with perfect index (0 mismatches)",
-            "max": 100,
-            "min": 0,
-            "scale": "RdYlGn",
-            "suffix": "%",
+        headers: Dict[str, ColumnDict] = {
+            "total_yield": {
+                "title": f"{config.base_count_prefix} Total Yield",
+                "description": f"Number of bases ({config.base_count_desc})",
+                "scale": "Greens",
+                "shared_key": "base_count",
+            },
+            "total": {
+                "title": f"{config.read_count_prefix} Total Clusters",
+                "description": f"Total number of clusters for this lane ({config.read_count_desc})",
+                "scale": "Blues",
+                "shared_key": "read_count",
+            },
+            "percent_Q30": {
+                "title": "% bases ≥ Q30",
+                "description": "Percentage of bases with greater than or equal to Q30 quality score",
+                "suffix": "%",
+                "max": 100,
+                "min": 0,
+                "scale": "RdYlGn",
+            },
+            "mean_qscore": {
+                "title": "Mean Quality",
+                "description": "Average phred quality score",
+                "min": 0,
+                "scale": "Spectral",
+            },
+            "percent_perfectIndex": {
+                "title": "% Perfect Index",
+                "description": "Percent of reads with perfect index (0 mismatches)",
+                "max": 100,
+                "min": 0,
+                "scale": "RdYlGn",
+                "suffix": "%",
+            },
         }
         table_config = {
             "namespace": "bcl2fastq",
             "id": "bcl2fastq-lane-stats-table",
-            "table_title": "bcl2fastq Lane Statistics",
+            "title": "bcl2fastq Lane Statistics",
             "col1_header": "Run ID - Lane",
-            "no_beeswarm": True,
         }
         return table.plot(self.bcl2fastq_bylane, headers, table_config)
 
-    def prepend_runid(self, runId, rest):
-        return str(runId) + " - " + str(rest)
+    @staticmethod
+    def prepend_runid(run_id: str, rest: str) -> str:
+        return str(run_id) + " - " + str(rest)
 
-    def get_bar_data_from_counts(self, counts):
+    @staticmethod
+    def get_bar_data_from_counts(data_by_flowcell):
         bar_data = {}
-        for key, value in counts.items():
-            bar_data[key] = {
-                "perfect": value["perfectIndex"],
-                "imperfect": value["total"] - value["perfectIndex"],
+        for flowcell_id, data in data_by_flowcell.items():
+            bar_data[flowcell_id] = {
+                "perfect": data["perfectIndex"],
+                "imperfect": data["total"] - data["perfectIndex"],
             }
-            if "undetermined" in value:
-                bar_data[key]["undetermined"] = value["undetermined"]
+            if "undetermined" in data:
+                bar_data[flowcell_id]["undetermined"] = data["undetermined"]
         return bar_data
 
-    def get_bar_data_from_undetermined(self, flowcells):
+    @staticmethod
+    def get_bar_data_from_undetermined(data_by_flowcell):
         """Get data to plot for undetermined barcodes."""
-        bar_data = defaultdict(dict)
+        bar_data: Dict[str, Dict[str, int]] = defaultdict(dict)
         # get undetermined barcodes for each lanes
-        for lane_id, lane in flowcells.items():
+        for flowcell_id, data in data_by_flowcell.items():
             try:
-                for barcode, count in islice(lane["unknown_barcodes"].items(), 20):
-                    bar_data[barcode][lane_id] = count
+                for barcode, count in islice(data["unknown_barcodes"].items(), 20):
+                    bar_data[barcode][flowcell_id] = count
             except AttributeError:
                 pass
 
-        # sort results
-        bar_data = OrderedDict(sorted(bar_data.items(), key=lambda x: sum(x[1].values()), reverse=True))
-        return OrderedDict((key, value) for key, value in islice(bar_data.items(), 20))
+        sorted_items = sorted(bar_data.items(), key=lambda kv: sum(kv[1].values()), reverse=True)
+        if len(sorted_items) > 20:
+            sorted_items = sorted_items[:20]
+        return dict(sorted_items)
