@@ -1,35 +1,23 @@
-#!/usr/bin/env python
-
-""" MultiQC module to parse output from DamageProfiler """
-
-from __future__ import print_function
-from collections import OrderedDict
-import logging
 import json
+import logging
 
+from multiqc.base_module import BaseMultiqcModule, ModuleNoSamplesFound
 from multiqc.plots import linegraph
-from multiqc.modules.base_module import BaseMultiqcModule
 
-# Initialise the logger
 log = logging.getLogger(__name__)
 
 
 class MultiqcModule(BaseMultiqcModule):
     def __init__(self):
-        """
-        damageprofiler module class
-        """
-
-        # Initialise the parent object
         super(MultiqcModule, self).__init__(
             name="DamageProfiler",
             anchor="damageprofiler",
             href="https://github.com/Integrative-Transcriptomics/DamageProfiler",
-            info="a tool to determine damage patterns on ancient DNA.",
+            info="DNA damage pattern retrieval for ancient DNA analysis",
+            doi="10.1093/bioinformatics/btab190",
         )
 
         # Init empty dictionaries
-
         self.threepGtoAfreq_data = dict()
         self.fivepCtoTfreq_data = dict()
         self.lgdist_fw_data = dict()
@@ -48,7 +36,7 @@ class MultiqcModule(BaseMultiqcModule):
         self.summary_metrics_data = self.ignore_samples(self.summary_metrics_data)
 
         if len(self.summary_metrics_data) == 0:
-            raise UserWarning
+            raise ModuleNoSamplesFound
 
         # Write parsed report data to a file
         self.write_data_file(self.summary_metrics_data, "multiqc_damageprofiler_metrics")
@@ -56,7 +44,7 @@ class MultiqcModule(BaseMultiqcModule):
         # Basic Stats Table, use generic function to add data to general table
         self.dmgprof_misinc_stats(self.threepGtoAfreq_data, "3 Prime", "G>A")
         self.dmgprof_misinc_stats(self.fivepCtoTfreq_data, "5 Prime", "C>T")
-        self.addSummaryMetrics(self.summary_metrics_data)
+        self.add_summary_metrics(self.summary_metrics_data)
 
         # Add plots
         if len(self.threepGtoAfreq_data) > 0:
@@ -101,14 +89,17 @@ class MultiqcModule(BaseMultiqcModule):
         try:
             parsed_json = json.load(f["f"])
         except Exception as e:
-            print(e)
-            log.warning("Could not parse DamageProfiler JSON: '{}'".format(f["fn"]))
+            log.warning(f"Could not parse DamageProfiler JSON: '{f['fn']}': {e}")
             return None
 
         # Get sample name from JSON first
         s_name = self.clean_s_name(parsed_json["metadata"]["sample_name"], f)
         self.add_data_source(f, s_name)
 
+        # Add version info
+        version = parsed_json["metadata"]["version"]
+        self.add_software_version(version, s_name)
+
         # Add 3' G to A data
         self.threepGtoAfreq_data[s_name] = parsed_json["dmg_3p"]
 
@@ -130,32 +121,32 @@ class MultiqcModule(BaseMultiqcModule):
         """Take the parsed stats from the DamageProfiler and add it to the
         basic stats table at the top of the report"""
 
-        headers = OrderedDict()
-        headers["{}1".format(readend)] = {
-            "id": "misinc-stats-1st-{}-{}".format(readend, substitution),
-            "title": "{} {} 1st base".format(readend, substitution),
-            "description": "{} 1st base substitution frequency for {}".format(readend, substitution),
-            "max": 100,
-            "min": 0,
-            "suffix": "%",
-            "scale": "YlGnBu",
-            "modify": lambda x: x * 100.0,
-        }
-        headers["{}2".format(readend)] = {
-            "id": "misinc-stats-2nd-{}-{}".format(readend, substitution),
-            "title": "{} {} 2nd base".format(readend, substitution),
-            "description": "{} 2nd base substitution frequency for {}".format(readend, substitution),
-            "max": 100,
-            "min": 0,
-            "suffix": "%",
-            "scale": "BuGn",
-            "hidden": True,
-            "modify": lambda x: x * 100.0,
+        headers = {
+            f"{readend}1": {
+                "rid": f"misinc-stats-1st-{readend}-{substitution}",
+                "title": f"{readend} {substitution} 1st base",
+                "description": f"{readend} 1st base substitution frequency for {substitution}",
+                "max": 100,
+                "min": 0,
+                "suffix": "%",
+                "scale": "YlGnBu",
+                "modify": lambda x: x * 100.0,
+            },
+            f"{readend}2": {
+                "rid": f"misinc-stats-2nd-{readend}-{substitution}",
+                "title": f"{readend} {substitution} 2nd base",
+                "description": f"{readend} 2nd base substitution frequency for {substitution}",
+                "max": 100,
+                "min": 0,
+                "suffix": "%",
+                "scale": "BuGn",
+                "hidden": True,
+                "modify": lambda x: x * 100.0,
+            },
         }
 
         # Create new small subset dictionary for entries (we need just the first two data (k,v) pairs from each report)
         # Only the first two parts are informative from both 3' and 5' ends of reads (1st, 2nd base damage pattern)
-        data = OrderedDict()
         dict_to_add = dict()
 
         for key in dict_to_plot.keys():
@@ -169,38 +160,37 @@ class MultiqcModule(BaseMultiqcModule):
 
         self.general_stats_addcols(dict_to_add, headers)
 
-    def addSummaryMetrics(self, dict_to_plot):
+    def add_summary_metrics(self, dict_to_plot):
         """Take the parsed stats from the DamageProfiler and add it to the
         basic stats table at the top of the report"""
 
-        headers = OrderedDict()
-
-        headers["std"] = {
-            "title": "Read length std. dev.",
-            "description": "Read length std. dev.",
-            "suffix": "bp",
-            "scale": "PuBu",
-            "format": "{:,.2f}",
-            "shared_key": "read_length",
-            "hidden": True,
-        }
-
-        headers["median"] = {
-            "title": "Median read length",
-            "description": "Median read length",
-            "suffix": "bp",
-            "scale": "YlGnBu",
-            "format": "{:,.2f}",
-            "shared_key": "read_length",
-        }
-        headers["mean_readlength"] = {
-            "title": "Mean read length",
-            "description": "Mean read length",
-            "suffix": "bp",
-            "scale": "PuBuGn",
-            "format": "{:,.2f}",
-            "shared_key": "read_length",
-            "hidden": True,
+        headers = {
+            "std": {
+                "title": "Read length std. dev.",
+                "description": "Read length std. dev.",
+                "suffix": "bp",
+                "scale": "PuBu",
+                "format": "{:,.2f}",
+                "shared_key": "read_length",
+                "hidden": True,
+            },
+            "median": {
+                "title": "Median read length",
+                "description": "Median read length",
+                "suffix": "bp",
+                "scale": "YlGnBu",
+                "format": "{:,.2f}",
+                "shared_key": "read_length",
+            },
+            "mean_readlength": {
+                "title": "Mean read length",
+                "description": "Mean read length",
+                "suffix": "bp",
+                "scale": "PuBuGn",
+                "format": "{:,.2f}",
+                "shared_key": "read_length",
+                "hidden": True,
+            },
         }
 
         self.general_stats_addcols(dict_to_plot, headers)
@@ -222,12 +212,13 @@ class MultiqcModule(BaseMultiqcModule):
             return None
 
         config = {
-            "id": "length-distribution-{}".format(orientation),
-            "title": "DamageProfiler: Read length distribution - {} ".format(orientation),
+            "id": f"length-distribution-{orientation}",
+            "title": f"DamageProfiler: Read length distribution - {orientation} ",
             "ylab": "Number of reads",
             "xlab": "Readlength (bp)",
-            "xDecimals": False,
-            "tt_label": "{point.y} reads of length {point.x}",
+            "x_decimals": False,
+            "tt_label": "{point.y} reads of length {point.x} bp",
+            "ysuffix": " reads",
             "ymin": 0,
             "xmin": 0,
         }
@@ -237,7 +228,6 @@ class MultiqcModule(BaseMultiqcModule):
     def threeprime_plot(self):
         """Generate a 3' G>A linegraph plot"""
 
-        data = dict()
         dict_to_add = dict()
         # Create tuples out of entries
         for key in self.threepGtoAfreq_data:
@@ -254,7 +244,8 @@ class MultiqcModule(BaseMultiqcModule):
             "title": "DamageProfiler: 3' G>A misincorporation plot",
             "ylab": "% G to A substituted",
             "xlab": "Nucleotide position from 3'",
-            "tt_label": "{point.y:.2f} % G>A misincorporations at nucleotide position {point.x}",
+            "tt_label": "{point.y:.2f}% G>A misincorporations at nucleotide position {point.x}",
+            "ysuffix": "%",
             "ymin": 0,
             "xmin": 1,
         }
@@ -265,7 +256,6 @@ class MultiqcModule(BaseMultiqcModule):
     def fiveprime_plot(self):
         """Generate a 5' C>T linegraph plot"""
 
-        data = dict()
         dict_to_add = dict()
         # Create tuples out of entries
         for key in self.fivepCtoTfreq_data:
@@ -281,7 +271,8 @@ class MultiqcModule(BaseMultiqcModule):
             "title": "DamageProfiler: 5' C>T misincorporation plot",
             "ylab": "% C to T substituted",
             "xlab": "Nucleotide position from 5'",
-            "tt_label": "{point.y:.2f} % C>T misincorporations at nucleotide position {point.x}",
+            "tt_label": "{point.y:.2f}% C>T misincorporations at nucleotide position {point.x}",
+            "ysuffix": "%",
             "ymin": 0,
             "xmin": 1,
         }
