@@ -1,36 +1,35 @@
-#!/usr/bin/env python
-
-""" MultiQC module to parse output from somalier """
-
-from __future__ import print_function
-from collections import OrderedDict, defaultdict
-from math import isnan, isinf
 import csv
 import logging
 import random
+from collections import defaultdict
+from math import isinf, isnan
+
 import spectra
 
+from multiqc.base_module import BaseMultiqcModule, ModuleNoSamplesFound
 from multiqc.plots import bargraph, heatmap, scatter, table
 from multiqc.utils import mqc_colour
-from multiqc.modules.base_module import BaseMultiqcModule
 
-# Initialise the logger
 log = logging.getLogger(__name__)
 
 
 class MultiqcModule(BaseMultiqcModule):
-    """
-    somalier module class, parses stderr logs.
-    """
-
     def __init__(self):
-
-        # Initialise the parent object
         super(MultiqcModule, self).__init__(
             name="Somalier",
             anchor="somalier",
             href="https://github.com/brentp/somalier",
-            info="calculates genotype :: pedigree correspondence checks from sketches derived from BAM/CRAM or VCF",
+            info="Genotype to pedigree correspondence checks from sketches derived from BAM/CRAM or VCF",
+            extra="""
+            Somalier can be used to find sample swaps or duplicates in cancer
+            projects, where there is often no jointly-called VCF across samples.
+
+            It is also extremely efficient and so can be used to find all-vs-all
+            relatedness estimates for thousands of samples.
+
+            It also outputs information on sex, depth, heterozgyosity, and ancestry
+            to be used for general QC.
+            """,
             doi="10.1186/s13073-020-00761-2",
         )
 
@@ -49,7 +48,7 @@ class MultiqcModule(BaseMultiqcModule):
                 for s_name_raw in parsed_data:
                     s_name = "*".join([self.clean_s_name(s, f) for s in s_name_raw.split("*")])
                     if s_name in self.somalier_data.keys():
-                        log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
+                        log.debug(f"Duplicate sample name found! Overwriting: {s_name}")
                     self.add_data_source(f, s_name)
                     self.somalier_data[s_name] = parsed_data[s_name_raw]
 
@@ -60,7 +59,7 @@ class MultiqcModule(BaseMultiqcModule):
                 for s_name_raw in parsed_data:
                     s_name = "*".join([self.clean_s_name(s, f) for s in s_name_raw.split("*")])
                     if s_name in self.somalier_data.keys():
-                        log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
+                        log.debug(f"Duplicate sample name found! Overwriting: {s_name}")
                     self.add_data_source(f, s_name)
                     self.somalier_data[s_name] = parsed_data[s_name_raw]
 
@@ -72,12 +71,13 @@ class MultiqcModule(BaseMultiqcModule):
         self.somalier_data = self.ignore_samples(self.somalier_data)
 
         if len(self.somalier_data) == 0:
-            raise UserWarning
+            raise ModuleNoSamplesFound
 
-        log.info("Found {} reports".format(len(self.somalier_data)))
+        log.info(f"Found {len(self.somalier_data)} reports")
 
-        # Write parsed report data to a file
-        self.write_data_file(self.somalier_data, "multiqc_somalier")
+        # Superfluous function call to confirm that it is used in this module
+        # Replace None with actual version if it is available
+        self.add_software_version(None)
 
         # Somalier Stats Table
         self.somalier_stats_table()
@@ -97,13 +97,17 @@ class MultiqcModule(BaseMultiqcModule):
 
         self.somalier_ancestry_pca_plot()
 
-    def parse_somalier_samples(self, f):
+        # Write parsed report data to a file
+        self.write_data_file(self.somalier_data, "multiqc_somalier")
+
+    @staticmethod
+    def parse_somalier_samples(f):
         """Go through log file looking for somalier output"""
         parsed_data = dict()
         headers = None
         sample_i = -100
-        for l in f["f"].splitlines():
-            s = l.split("\t")
+        for line in f["f"].splitlines():
+            s = line.split("\t")
             if headers is None:
                 s[0] = s[0].lstrip("#")
                 headers = s
@@ -120,35 +124,35 @@ class MultiqcModule(BaseMultiqcModule):
             return None
         return parsed_data
 
-    def parse_somalier_pairs_tsv(self, f):
+    @staticmethod
+    def parse_somalier_pairs_tsv(f):
         """Parse csv output from somalier"""
         parsed_data = dict()
         headers = None
         s_name_idx = None
-        for l in f["f"].splitlines():
-            s = l.lstrip("#").split("\t")
+        for line in f["f"].splitlines():
+            s = line.lstrip("#").split("\t")
             if headers is None:
                 headers = s
                 try:
                     s_name_idx = [headers.index("sample_a"), headers.index("sample_b")]
                 except ValueError:
-                    log.warning("Could not find sample name in somalier output: {}".format(f["fn"]))
+                    log.warning(f"Could not find sample name in somalier output: {f['fn']}")
                     return None
             else:
+                if s_name_idx is None:
+                    continue
                 s_name = "*".join([s[idx] for idx in s_name_idx])  # not safe to hard code, but works
                 parsed_data[s_name] = dict()
                 for i, v in enumerate(s):
                     if i not in s_name_idx:  # Skip if (i == 0 or 1); i.e. sample_a, sample_b
                         if isnan(float(v)) or isinf(float(v)):
-                            # TODO: find better solution
-                            log.debug("Found Inf or NaN value. Overwriting with -2.")
-                            v = -2
-                        try:
+                            # Inf or NaN indicate the absence of data
+                            v = None
+                        else:
                             # add the pattern as a suffix to key
-                            parsed_data[s_name][headers[i]] = float(v)
-                        except ValueError:
-                            # add the pattern as a suffix to key
-                            parsed_data[s_name][headers[i]] = v
+                            v = float(v)
+                        parsed_data[s_name][headers[i]] = v
 
         if len(parsed_data) == 0:
             return None
@@ -163,7 +167,7 @@ class MultiqcModule(BaseMultiqcModule):
         bg_pc2 = []
         bg_ancestry = []
 
-        reader = csv.DictReader(f["f"], dialect="excel-tab")
+        reader: csv.DictReader = csv.DictReader(f["f"], dialect="excel-tab")
         idx = "#sample_id"
 
         # check file not empty, else parse file
@@ -214,185 +218,183 @@ class MultiqcModule(BaseMultiqcModule):
                     if s_name in self.somalier_data.keys():
                         intersect_keys = parsed_data[s_name_raw].keys() & self.somalier_data.keys()
                         if len(intersect_keys) > 0:
-                            log.debug(
-                                "Duplicate sample name found! Overwriting: {} : {}".format(s_name, intersect_keys)
-                            )
+                            log.debug(f"Duplicate sample name found! Overwriting: {s_name} : {intersect_keys}")
                     self.add_data_source(f, s_name)
                     try:
                         self.somalier_data[s_name].update(parsed_data[s_name_raw])
                     except KeyError:
                         self.somalier_data[s_name] = parsed_data[s_name_raw]
         else:
-            log.warning("Detected empty file: {}".format(f["fn"]))
+            log.warning(f"Detected empty file: {f['fn']}")
 
     def somalier_stats_table(self):
         """Add data to somalier stats table
 
         Bigger table within the somalier module, showing more stats"""
 
-        headers = OrderedDict()
-
-        headers["phenotype"] = {
-            "title": "Phenotype",
-            "description": "Sample's phenotype from pedigree info",
-            "hidden": True,
-        }
-        headers["original_pedigree_sex"] = {
-            "title": "Sex",
-            "description": "Sample's sex from pedigree info",
-            "scale": False,
-        }
-        headers["paternal_id"] = {
-            "title": "Father ID",
-            "description": "ID of sample's father ",
-            "scale": False,
-            "hidden": True,
-        }
-        headers["maternal_id"] = {
-            "title": "Mother ID",
-            "description": "ID of sample's mother",
-            "scale": False,
-            "hidden": True,
-        }
-        headers["family_id"] = {
-            "title": "Family ID",
-            "description": "ID of sample's family",
-            "scale": False,
-            "hidden": True,
-        }
-        headers["sex"] = {
-            "title": "Inferred sex",
-            "description": "Sample's inferred sex",
-            "scale": False,
-            "hidden": True,
-        }
-        headers["ancestry"] = {"title": "Ancestry", "description": "Most probable ancestry background", "scale": False}
-        headers["p_ancestry"] = {
-            "title": "P(Ancestry)",
-            "description": "Ancestry probablitty",
-            "max": 1,
-            "min": 0,
-            "scale": "RdYlGn",
-            "format": "{:,.2f}",
-        }
-        headers["n_het"] = {
-            "title": "HetVar",
-            "description": "Heterozygous variants",
-            "shared_key": "variant_count",
-            "format": "{:,.0f}",
-        }
-        headers["n_hom_ref"] = {
-            "title": "HomRefVar",
-            "description": "Homozygous reference variants",
-            "shared_key": "variant_count",
-            "format": "{:,.0f}",
-            "hidden": True,
-        }
-        headers["n_hom_alt"] = {
-            "title": "HomAltVar",
-            "description": "Homozygous alternate variants",
-            "shared_key": "variant_count",
-            "format": "{:,.0f}",
-            "hidden": True,
-        }
-        headers["n_unknown"] = {"title": "NA sites", "description": "Unknown sites", "format": "{:,.0f}"}
-        headers["depth_mean"] = {
-            "title": "Mean depth",
-            "description": "Mean depth of all sites",
-            "scale": "RdYlGn",
-            "suffix": " X",
-            "hidden": True,
-        }
-        headers["depth_sd"] = {
-            "title": "Depth std",
-            "description": "Depth's standard deviation of all sites",
-            "scale": "RdYlGn",
-            "hidden": True,
-        }
-        headers["gt_depth_mean"] = {
-            "title": "Sites depth",
-            "description": "Mean depth of genotyped sites",
-            "scale": "RdYlGn",
-            "suffix": " X",
-        }
-        headers["gt_depth_sd"] = {
-            "title": "Genot depth std",
-            "description": "Depth's standard deviation of genotype sites",
-            "scale": "RdYlGn",
-            "suffix": " X",
-            "hidden": True,
-        }
-        headers["ab_mean"] = {
-            "title": "Allele balance",
-            "description": "Mean allele balance",
-            "scale": "RdYlGn",
-        }
-        headers["ab_std"] = {
-            "title": "Allele balance std",
-            "description": "Standard deviation of allele balance",
-            "scale": "RdYlGn",
-            "hidden": True,
-        }
-        headers["p_middling_ab"] = {
-            "title": "Allele balance < 0.2, > 0.8",
-            "description": "Proportion of sites with allele balance < 0.2 or > 0.8",
-            "max": 1,
-            "min": 0,
-            "scale": "RdYlGn",
-            "format": "{:,.2f}",
-        }
-        headers["X_het"] = {
-            "title": "HetVar X",
-            "description": "Heterozygous variants on X chromosome",
-            "shared_key": "variant_count_xy",
-            "format": "{:,.0f}",
-        }
-        headers["X_hom_ref"] = {
-            "title": "HomRefVar X",
-            "description": "Homozygous reference variants on X chromosome",
-            "shared_key": "variant_count_xy",
-            "format": "{:,.0f}",
-            "hidden": True,
-        }
-        headers["X_hom_alt"] = {
-            "title": "HomAltVar X",
-            "description": "Homozygous alternate variants on X chromosome",
-            "shared_key": "variant_count_xy",
-            "format": "{:,.0f}",
-            "hidden": True,
-        }
-        headers["X_n"] = {
-            "title": "Sites X",
-            "description": "Total sites on X chromosome",
-            "shared_key": "variant_count_xy",
-            "format": "{:,.0f}",
-            "hidden": True,
-        }
-        headers["X_depth_mean"] = {
-            "title": "Mean depth X",
-            "description": "Mean depth of sites on X chromosome",
-            "scale": "RdYlGn",
-            "suffix": " X",
-        }
-        headers["Y_n"] = {
-            "title": "Sites Y",
-            "description": "Total sites on Y chromosome",
-            "shared_key": "variant_count_xy",
-            "format": "{:,.0f}",
-            "hidden": True,
-        }
-        headers["Y_depth_mean"] = {
-            "title": "Mean depth Y",
-            "description": "Mean depth of sites on Y chromosome",
-            "scale": "RdYlGn",
-            "suffix": " X",
+        headers = {
+            "phenotype": {
+                "title": "Phenotype",
+                "description": "Sample's phenotype from pedigree info",
+                "hidden": True,
+            },
+            "original_pedigree_sex": {
+                "title": "Sex",
+                "description": "Sample's sex from pedigree info",
+                "scale": False,
+            },
+            "paternal_id": {
+                "title": "Father ID",
+                "description": "ID of sample's father ",
+                "scale": False,
+                "hidden": True,
+            },
+            "maternal_id": {
+                "title": "Mother ID",
+                "description": "ID of sample's mother",
+                "scale": False,
+                "hidden": True,
+            },
+            "family_id": {
+                "title": "Family ID",
+                "description": "ID of sample's family",
+                "scale": False,
+                "hidden": True,
+            },
+            "sex": {
+                "title": "Inferred sex",
+                "description": "Sample's inferred sex",
+                "scale": False,
+                "hidden": True,
+            },
+            "ancestry": {"title": "Ancestry", "description": "Most probable ancestry background", "scale": False},
+            "p_ancestry": {
+                "title": "P(Ancestry)",
+                "description": "Ancestry probability",
+                "max": 1,
+                "min": 0,
+                "scale": "RdYlGn",
+                "format": "{:,.2f}",
+            },
+            "n_het": {
+                "title": "HetVar",
+                "description": "Heterozygous variants",
+                "shared_key": "variant_count",
+                "format": "{:,.0f}",
+            },
+            "n_hom_ref": {
+                "title": "HomRefVar",
+                "description": "Homozygous reference variants",
+                "shared_key": "variant_count",
+                "format": "{:,.0f}",
+                "hidden": True,
+            },
+            "n_hom_alt": {
+                "title": "HomAltVar",
+                "description": "Homozygous alternate variants",
+                "shared_key": "variant_count",
+                "format": "{:,.0f}",
+                "hidden": True,
+            },
+            "n_unknown": {"title": "NA sites", "description": "Unknown sites", "format": "{:,.0f}"},
+            "depth_mean": {
+                "title": "Mean depth",
+                "description": "Mean depth of all sites",
+                "scale": "RdYlGn",
+                "suffix": " X",
+                "hidden": True,
+            },
+            "depth_sd": {
+                "title": "Depth std",
+                "description": "Depth's standard deviation of all sites",
+                "scale": "RdYlGn",
+                "hidden": True,
+            },
+            "gt_depth_mean": {
+                "title": "Sites depth",
+                "description": "Mean depth of genotyped sites",
+                "scale": "RdYlGn",
+                "suffix": " X",
+            },
+            "gt_depth_sd": {
+                "title": "Genot depth std",
+                "description": "Depth's standard deviation of genotype sites",
+                "scale": "RdYlGn",
+                "suffix": " X",
+                "hidden": True,
+            },
+            "ab_mean": {
+                "title": "Allele balance",
+                "description": "Mean allele balance",
+                "scale": "RdYlGn",
+            },
+            "ab_std": {
+                "title": "Allele balance std",
+                "description": "Standard deviation of allele balance",
+                "scale": "RdYlGn",
+                "hidden": True,
+            },
+            "p_middling_ab": {
+                "title": "Allele balance < 0.2, > 0.8",
+                "description": "Proportion of sites with allele balance < 0.2 or > 0.8",
+                "max": 1,
+                "min": 0,
+                "scale": "RdYlGn",
+                "format": "{:,.2f}",
+            },
+            "X_het": {
+                "title": "HetVar X",
+                "description": "Heterozygous variants on X chromosome",
+                "shared_key": "variant_count_xy",
+                "format": "{:,.0f}",
+            },
+            "X_hom_ref": {
+                "title": "HomRefVar X",
+                "description": "Homozygous reference variants on X chromosome",
+                "shared_key": "variant_count_xy",
+                "format": "{:,.0f}",
+                "hidden": True,
+            },
+            "X_hom_alt": {
+                "title": "HomAltVar X",
+                "description": "Homozygous alternate variants on X chromosome",
+                "shared_key": "variant_count_xy",
+                "format": "{:,.0f}",
+                "hidden": True,
+            },
+            "X_n": {
+                "title": "Sites X",
+                "description": "Total sites on X chromosome",
+                "shared_key": "variant_count_xy",
+                "format": "{:,.0f}",
+                "hidden": True,
+            },
+            "X_depth_mean": {
+                "title": "Mean depth X",
+                "description": "Mean depth of sites on X chromosome",
+                "scale": "RdYlGn",
+                "suffix": " X",
+            },
+            "Y_n": {
+                "title": "Sites Y",
+                "description": "Total sites on Y chromosome",
+                "shared_key": "variant_count_xy",
+                "format": "{:,.0f}",
+                "hidden": True,
+            },
+            "Y_depth_mean": {
+                "title": "Mean depth Y",
+                "description": "Mean depth of sites on Y chromosome",
+                "scale": "RdYlGn",
+                "suffix": " X",
+            },
         }
 
         t_config = {
             "id": "somalier_stats",
             "namespace": "Somalier",
             "title": "Somalier: Statistics",
-            "no_beeswarm": True,
+            "no_violin": True,
             "raw_data_fn": "multiqc_somalier_stats",
         }
 
@@ -404,66 +406,79 @@ class MultiqcModule(BaseMultiqcModule):
         )
 
     def somalier_relatedness_plot(self):
-        data = dict()
         alpha = 0.6
-        relatedness_colours = {
-            0: ["Unrelated", "rgba(74, 124, 182, {})".format(alpha)],
-            0.49: ["Sib-sib", "rgba(243, 123, 40, {})".format(alpha)],
-            0.5: ["Parent-child", "rgba(159, 84, 47, {})".format(alpha)],
+        relatedness_groups = {
+            0: {
+                "name": "Unrelated",
+                "color": f"rgba(74, 124, 182, {alpha})",
+            },
+            0.49: {
+                "name": "Sib-sib",
+                "color": f"rgba(243, 123, 40, {alpha})",
+            },
+            0.5: {
+                "name": "Parent-child",
+                "color": f"rgba(159, 84, 47, {alpha})",
+            },
         }
 
         # Get index colour scale
         cscale = mqc_colour.mqc_colour_scale()
         extra_colours = cscale.get_colours("Dark2")
         extra_colours = _make_col_alpha(extra_colours, alpha)
-
         extra_colour_idx = 0
-        for s_name, d in self.somalier_data.items():
-            if "ibs0" in d and "ibs2" in d:
-                data[s_name] = {"x": d["ibs0"], "y": d["ibs2"]}
-            if "relatedness" in d:
-                relatedness = d["expected_relatedness"]
-                # -1 is not the same family, 0 is same family but unreleaed
-                # @brentp says he usually bundles them together
-                if relatedness == -1:
-                    relatedness = 0
+        data = dict()
+        for pair, d in self.somalier_data.items():
+            if "expected_relatedness" not in d:
+                continue
 
-                # New unique value that we've not seen before
-                if relatedness not in relatedness_colours:
-                    relatedness_colours[relatedness] = [str(relatedness), extra_colours[extra_colour_idx]]
-                    extra_colour_idx += 0
-                    if extra_colour_idx > len(extra_colours):
-                        extra_colour_idx = 0
+            relatedness = d["expected_relatedness"]
+            # -1 is not the same family, 0 is same family but unrelated
+            # @brentp says he usually bundles them together
+            if relatedness == -1:
+                relatedness = 0
 
-                # Assign colour
-                data[s_name]["color"] = relatedness_colours[relatedness][1]
+            # New unique value that we've not seen before
+            if relatedness not in relatedness_groups:
+                relatedness_groups[relatedness] = {
+                    "name": str(relatedness),
+                    "color": extra_colours[extra_colour_idx],
+                }
+                extra_colour_idx += 0
+                if extra_colour_idx > len(extra_colours):
+                    extra_colour_idx = 0
 
-        if len(data) > 0:
-            pconfig = {
-                "id": "somalier_relatedness_plot",
-                "title": "Somalier: Sample Shared Allele Rates (IBS)",
-                "xlab": "IBS0 (no alleles shared)",
-                "ylab": "IBS2 (both alleles shared)",
-                "marker_line_width": 0,
+            data[pair] = {
+                "x": d["ibs0"],
+                "y": d["ibs2"],
+                "color": relatedness_groups[relatedness]["color"],
+                "group": relatedness_groups[relatedness]["name"],
             }
 
-            colours_legend = ""
-            for val in sorted(relatedness_colours.keys()):
-                name, col_rgb = relatedness_colours[val]
-                colours_legend += '<span style="color:{}">{}</span>, '.format(
-                    col_rgb.replace(str(alpha), "1.0"), name, val
-                )
+        if len(data) == 0:
+            return
 
-            self.add_section(
-                name="Relatedness",
-                anchor="somalier-relatedness",
-                description="""
-                Shared allele rates between sample pairs.
-                Points are coloured by degree of expected-relatedness: {}""".format(
-                    colours_legend
-                ),
-                plot=scatter.plot(data, pconfig),
-            )
+        pconfig = {
+            "id": "somalier_relatedness_plot",
+            "title": "Somalier: Sample Shared Allele Rates (IBS)",
+            "xlab": "IBS0 (no alleles shared)",
+            "ylab": "IBS2 (both alleles shared)",
+            "marker_line_width": 0,
+        }
+
+        colours_legend = ""
+        for rel, group in sorted(relatedness_groups.items()):
+            col = group["color"].replace(str(alpha), "1.0")
+            colours_legend += f'<span style="color:{col}">{group["name"]}</span>, '
+
+        self.add_section(
+            name="Relatedness",
+            anchor="somalier-relatedness",
+            description=f"""
+            Shared allele rates between sample pairs.
+            Points are coloured by degree of expected relatedness: {colours_legend}""",
+            plot=scatter.plot(data, pconfig),
+        )
 
     def somalier_relatedness_heatmap_plot(self):
         # inspiration: MultiQC/modules/vcftools/relatedness2.py
@@ -476,8 +491,8 @@ class MultiqcModule(BaseMultiqcModule):
                 a, b = s_name.split("*")
                 labels.add(a)
                 labels.add(b)
-                rels[a][b] = rels[b][a] = float(d["relatedness"])
-                rels[a][a] = rels[b][b] = float(1)
+                rels[a][b] = rels[b][a] = d["relatedness"]
+                rels[a][a] = rels[b][b] = 1.0
 
         # impose alphabetical order and avoid json serialisation errors in utils.report
         labels = sorted(labels)
@@ -488,7 +503,7 @@ class MultiqcModule(BaseMultiqcModule):
                 try:
                     line.append(rels[x][y])
                 except KeyError:
-                    line.append(-2)
+                    line.append(None)
             data.append(line)
 
         if len(data) > 0:
@@ -528,12 +543,13 @@ class MultiqcModule(BaseMultiqcModule):
                 "id": "somalier_het_check_plot",
                 "title": "Somalier: Sample Observed Heterozygosity",
                 "xlab": "Mean depth",
+                "xsuffix": "x",
                 "ylab": "Standard deviation of allele-balance",
             }
 
             self.add_section(
                 name="Heterozygosity",
-                description="Standard devation of heterozygous allele balance against mean depth.",
+                description="Standard deviation of heterozygous allele balance against mean depth.",
                 helptext="A high standard deviation in allele balance suggests contamination.",
                 anchor="somalier-hetcheck",
                 plot=scatter.plot(data, pconfig),
@@ -550,14 +566,14 @@ class MultiqcModule(BaseMultiqcModule):
                 else:
                     y = 2 * d["X_depth_mean"] / d["gt_depth_mean"]
                 data[s_name] = {
-                    "x": (random.random() - 0.5) * 0.1 + sex_index.get(d["original_pedigree_sex"], 2),
+                    "x": sex_index.get(d["original_pedigree_sex"], 2) + (random.random() - 0.5) * 0.1,
                     "y": y,
                 }
 
         if len(data) > 0:
             pconfig = {
                 "id": "somalier_sex_check_plot",
-                "title": "Somalier: Sample Predicted Sex",
+                "title": "Somalier: Provided Sex vs. Scaled X Depth",
                 "xlab": "Sex from pedigree",
                 "ylab": "Scaled mean depth on X",
                 "categories": ["Female", "Male", "Unknown"],
@@ -565,8 +581,8 @@ class MultiqcModule(BaseMultiqcModule):
 
             self.add_section(
                 name="Sex",
-                description="Predicted sex against scaled depth on X",
-                helptext="Higher values of depth, low values suggest male.",
+                description="Provided sex against scaled depth on X",
+                helptext="Higher depth values suggest female, lower values suggest male.",
                 anchor="somalier-sexcheck",
                 plot=scatter.plot(data, pconfig),
             )
@@ -574,7 +590,7 @@ class MultiqcModule(BaseMultiqcModule):
     def somalier_ancestry_barplot(self):
         data = dict()
         c_scale = mqc_colour.mqc_colour_scale(name="Paired").colours
-        cats = OrderedDict()
+        cats = dict()
         anc_cats = self.somalier_ancestry_cats
 
         # use Paired color scale, unless number of categories exceed colors
@@ -593,7 +609,7 @@ class MultiqcModule(BaseMultiqcModule):
         for s_name, d in self.somalier_data.items():
             # ensure that only relevant items are added,
             # i.e. only ancestry category values
-            ls = {k: v for k, v in d.items() if (k in self.somalier_ancestry_cats)}
+            ls = {k: v * 100.0 for k, v in d.items() if (k in self.somalier_ancestry_cats)}
             if len(ls) > 0:  # only add dict, if it contains values
                 data[s_name] = ls
 
@@ -601,9 +617,10 @@ class MultiqcModule(BaseMultiqcModule):
             pconfig = {
                 "id": "somalier_ancestry_barplot",
                 "title": "Somalier: Sample Predicted Ancestry Proportions",
-                "cpswitch_c_active": False,
+                "cpswitch": False,
                 "hide_zero_cats": False,
                 "ylab": "Predicted Ancestry",
+                "tt_suffix": "%",
             }
 
             self.add_section(
@@ -620,7 +637,34 @@ class MultiqcModule(BaseMultiqcModule):
             )
 
     def somalier_ancestry_pca_plot(self):
-        data = OrderedDict()
+        data = dict()
+
+        # add background
+        # N.B. this must be done after samples to have samples on top
+        d = self.somalier_background_pcs.pop("background_pcs", {})
+        if d:
+            # generate color scale to match the number of categories
+            c_scale = mqc_colour.mqc_colour_scale(name="Paired").colours
+            cats = self.somalier_ancestry_cats
+            ancestry_colors = dict(zip(cats, c_scale[: len(cats)]))
+            default_background_color = "rgba(255,192,203,0.3)"
+
+            # Make colours semi-transparent
+            ancestry_colors = dict(zip(ancestry_colors.keys(), _make_col_alpha(ancestry_colors.values(), 0.3)))
+
+            background = [
+                {
+                    "x": pc1,
+                    "y": pc2,
+                    "color": ancestry_colors.get(ancestry, default_background_color),
+                    "name": ancestry,
+                    "marker_size": 3,
+                    "marker_line_width": 0,
+                    "annotate": False,
+                }
+                for pc1, pc2, ancestry in zip(d["PC1"], d["PC2"], d["ancestry"])
+            ]
+            data["background"] = background
 
         # cycle over samples and add PC coordinates to data dict
         for s_name, d in self.somalier_data.items():
@@ -631,25 +675,6 @@ class MultiqcModule(BaseMultiqcModule):
                     "color": "rgba(0, 0, 0, 0.6)",
                 }
 
-        # add background
-        # N.B. this must be done after samples to have samples on top
-        d = self.somalier_background_pcs.pop("background_pcs", {})
-        if d:
-            # generate color scale to match the number of categories
-            c_scale = mqc_colour.mqc_colour_scale(name="Paired").colours
-            cats = self.somalier_ancestry_cats
-            ancestry_colors = dict(zip(cats, c_scale[: len(cats)]))
-            default_background_color = "rgb(255,192,203,0.3)"
-
-            # Make colours semi-transparent
-            ancestry_colors = dict(zip(ancestry_colors.keys(), _make_col_alpha(ancestry_colors.values(), 0.3)))
-
-            background = [
-                {"x": pc1, "y": pc2, "color": ancestry_colors.get(ancestry, default_background_color), "name": ancestry}
-                for pc1, pc2, ancestry in zip(d["PC1"], d["PC2"], d["ancestry"])
-            ]
-            data["background"] = background
-
         # generate section and plot
         if len(data) > 0:
             pconfig = {
@@ -658,7 +683,6 @@ class MultiqcModule(BaseMultiqcModule):
                 "xlab": "PC1",
                 "ylab": "PC2",
                 "marker_size": 5,
-                "marker_line_width": 0,
             }
 
             self.add_section(
@@ -677,10 +701,16 @@ class MultiqcModule(BaseMultiqcModule):
 
 
 def _make_col_alpha(cols, alpha):
-    """Take a HTML colour value and return a rgba string with alpha"""
+    """Take an HTML colour value and return a rgba string with alpha"""
     cols_return = []
     for col in cols:
-        col_srgb = spectra.html(col)
-        cols_rgb = [c * 255.0 for c in col_srgb.clamped_rgb]
-        cols_return.append("rgba({},{},{},{})".format(*cols_rgb, alpha))
+        # Check if already in rgba format
+        if col.startswith("rgba("):
+            # Already has alpha, just return as-is
+            cols_return.append(col)
+        else:
+            # Convert to rgba
+            col_srgb = spectra.html(col)
+            cols_rgb = [c * 255.0 for c in col_srgb.clamped_rgb]
+            cols_return.append("rgba({},{},{},{})".format(*cols_rgb, alpha))
     return cols_return
