@@ -23,7 +23,7 @@ from multiqc.core.order_modules_and_sections import order_modules_and_sections
 from multiqc.core.update_config import ClConfig, update_config
 from multiqc.core.version_check import check_version
 from multiqc.core.write_results import write_results
-from multiqc.utils import util_functions
+from multiqc.utils import config_schema, util_functions
 from multiqc.validation import ModuleConfigValidationError
 
 logger = logging.getLogger(__name__)
@@ -70,6 +70,7 @@ click.rich_click.OPTION_GROUPS = {
             "options": [
                 "--module",
                 "--exclude",
+                "--require-logs",
             ],
         },
         {
@@ -105,6 +106,7 @@ click.rich_click.OPTION_GROUPS = {
                 "--data-format",
                 "--zip-data-dir",
                 "--no-report",
+                "--clean-up",
                 "--pdf",
             ],
         },
@@ -113,9 +115,9 @@ click.rich_click.OPTION_GROUPS = {
             "options": [
                 "--verbose",
                 "--quiet",
+                "--no-version-check",
                 "--strict",
                 "--development",
-                "--require-logs",
                 "--profile-runtime",
                 "--profile-memory",
                 "--no-megaqc-upload",
@@ -124,6 +126,21 @@ click.rich_click.OPTION_GROUPS = {
                 "--help",
             ],
         },
+        {
+            "name": "AI Features",
+            "options": [
+                "--ai-summary",
+                "--ai-summary-full",
+                "--ai-provider",
+                "--no-ai",
+            ],
+        },
+        {
+            "name": "Check Config",
+            "options": [
+                "--check-config",
+            ],
+        },
     ],
 }
 
@@ -215,6 +232,14 @@ click.rich_click.OPTION_GROUPS = {
     metavar="GLOB EXPRESSION",
     help="Ignore sample names",
 )
+@click.option(
+    "--only-samples",
+    "only_samples",
+    type=str,
+    multiple=True,
+    metavar="GLOB EXPRESSION",
+    help="Only include sample names matching the given glob expression",
+)
 @click.option(
     "--ignore-symlinks",
     "ignore_symlinks",
@@ -434,8 +459,56 @@ click.rich_click.OPTION_GROUPS = {
     default=None,
     help="Disable checking the latest MultiQC version on the server",
 )
+@click.option(
+    "--ai",
+    "--ai-summary",
+    "ai_summary",
+    is_flag=True,
+    default=None,
+    help="Generate an AI summary of the report",
+)
+@click.option(
+    "--ai-summary-full",
+    "ai_summary_full",
+    is_flag=True,
+    default=None,
+    help="Generate a detailed AI summary of the report",
+)
+@click.option(
+    "--ai-provider",
+    type=click.Choice(config_schema.AiProviderLiteral.__args__),  # type: ignore
+    help=f"Select AI provider for report summarization. [dim yellow](Default: {config.ai_provider})",
+)
+@click.option(
+    "--ai-model",
+    type=str,
+    help="Select AI model to use for report summarization",
+)
+@click.option(
+    "--ai-custom-endpoint",
+    type=str,
+    help="Custom AI endpoint to use with OpenAI API",
+)
+@click.option(
+    "--ai-custom-context-window",
+    type=int,
+    help="Custom context window to use with OpenAI API (default: 128000)",
+)
+@click.option(
+    "--no-ai",
+    "no_ai",
+    is_flag=True,
+    default=None,
+    help="Disable AI toolbox and buttons in the report",
+)
+@click.option(
+    "--check-config",
+    is_flag=True,
+    default=False,
+    help="Check a MultiQC configuration file for errors and exit.",
+)
 @click.version_option(config.version, prog_name="multiqc")
-def run_cli(analysis_dir: Tuple[str], clean_up: bool, **kwargs):
+def run_cli(analysis_dir: Tuple[str], clean_up: bool, check_config: bool, **kwargs):
     # Main MultiQC run command for use with the click command line, complete with all click function decorators.
     # To make it easy to use MultiQC within notebooks and other locations that don't need click, we simply pass the
     # parsed variables on to a vanilla python function.
@@ -450,6 +523,18 @@ def run_cli(analysis_dir: Tuple[str], clean_up: bool, **kwargs):
     For example, to run in the current working directory, use '[blue bold]multiqc .[/]'
     """
 
+    # New: Check if this is a config check run
+    if check_config:
+        if len(analysis_dir) == 1:
+            config_file = analysis_dir[0]
+            from multiqc.core.config_check import check_config_file
+
+            check_config_file(config_file)
+            sys.exit(0)
+        else:
+            logger.error("Please specify a single config file with --check-config")
+            sys.exit(1)
+
     cl_config_kwargs = {k: v for k, v in kwargs.items() if k in ClConfig.model_fields}
     other_fields = {k: v for k, v in kwargs.items() if k not in ClConfig.model_fields}
     cfg = ClConfig(**cl_config_kwargs, unknown_options=other_fields)
@@ -469,14 +554,22 @@ class RunResult:
 
     * appropriate error code (e.g. 1 if a module broke, 0 on success)
     * error message if a module broke
+    * optionally, the HTML report content if return_html=True was specified
     """
 
-    def __init__(self, sys_exit_code: int = 0, message: str = ""):
+    def __init__(self, sys_exit_code: int = 0, message: str = "", html_content: Optional[str] = None):
         self.sys_exit_code = sys_exit_code
         self.message = message
+        self.html_content = html_content
 
 
-def run(*analysis_dir, clean_up: bool = True, cfg: Optional[ClConfig] = None, interactive: bool = True) -> RunResult:
+def run(
+    *analysis_dir,
+    clean_up: bool = True,
+    cfg: Optional[ClConfig] = None,
+    interactive: bool = True,
+    return_html: bool = False,
+) -> RunResult:
     """
     MultiQC aggregates results from bioinformatics analyses across many samples into a single report.
 
@@ -487,6 +580,16 @@ def run(*analysis_dir, clean_up: bool = True, cfg: Optional[ClConfig] = None, in
     To run, supply with one or more directory to scan for analysis results.
     To run here, use 'multiqc .'
 
+    Args:
+        *analysis_dir: Directories to scan for analysis results
+        clean_up: Whether to clean up temporary files
+        cfg: Configuration object
+        interactive: Whether to run in interactive mode
+        return_html: If True, return HTML report content in RunResult.html_content
+
+    Returns:
+        RunResult: Contains exit code, message, and optionally HTML content
+
     See http://multiqc.info for more details.
     """
 
@@ -512,6 +615,7 @@ def run(*analysis_dir, clean_up: bool = True, cfg: Optional[ClConfig] = None, in
     report.multiqc_command = " ".join(sys.argv)
     logger.debug(f"Command used: {report.multiqc_command}")
 
+    html_content = None
     try:
         mod_dicts_in_order = file_search()
 
@@ -519,7 +623,7 @@ def run(*analysis_dir, clean_up: bool = True, cfg: Optional[ClConfig] = None, in
 
         order_modules_and_sections()
 
-        write_results()
+        html_content = write_results(return_html=return_html)
 
     except NoAnalysisFound as e:
         logger.warning(f"{e.message}. Cleaning up…")
@@ -538,10 +642,14 @@ def run(*analysis_dir, clean_up: bool = True, cfg: Optional[ClConfig] = None, in
         return RunResult(message=e.message, sys_exit_code=e.sys_exit_code)
 
     except KeyboardInterrupt:
-        logger.critical(
-            "User Cancelled Execution!\n{eq}\n{tb}{eq}\n".format(eq=("=" * 60), tb=traceback.format_exc())
-            + "User Cancelled Execution!\nExiting MultiQC..."
-        )
+        if config.verbose:
+            msg = (
+                "User Cancelled Execution!\n{eq}\n{tb}{eq}\n".format(eq=("=" * 60), tb=traceback.format_exc())
+                + "User Cancelled Execution!\nExiting MultiQC..."
+            )
+        else:
+            msg = "User Cancelled Execution! Exiting MultiQC..."
+        logger.critical(msg)
         return RunResult(sys_exit_code=1)
 
     else:
@@ -569,8 +677,11 @@ def run(*analysis_dir, clean_up: bool = True, cfg: Optional[ClConfig] = None, in
             logger.error(f"{len(report.lint_errors)} linting errors:\n" + "\n".join(report.lint_errors))
             sys_exit_code = 1
 
-        logger.info("MultiQC complete") if sys_exit_code == 0 else logger.error("MultiQC complete with errors")
-        return RunResult(sys_exit_code=sys_exit_code)
+        if sys_exit_code == 0:
+            logger.info("MultiQC complete")
+        else:
+            logger.error("MultiQC complete with errors")
+        return RunResult(sys_exit_code=sys_exit_code, html_content=html_content if return_html else None)
 
     finally:
         if clean_up:
@@ -628,4 +739,4 @@ def _print_intro_with_coloredlogs():
 
 def _print_intro_with_rich():
     if log_and_rich.rich_console is not None:
-        log_and_rich.rich_console.print(f"\n{log_and_rich.rich_click.rich_click.HEADER_TEXT}\n")
+        log_and_rich.rich_console.print(f"\n{click.rich_click.HEADER_TEXT}\n")
